==> expense_tracker-1.2.1.tar.gz: expense_tracker/src/Form/EtTransactionViewForm.php <==
<?php

namespace Drupal\expense_tracker\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\BaseFormIdInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\expense_tracker\EtTransactionInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Displays banned IP addresses.
 */
class EtTransactionViewForm extends FormBase implements BaseFormIdInterface {

  /**
   * The EtTransaction of the form.
   *
   * @var \Drupal\expense_tracker\EtTransactionInterface
   */
  protected $et_transaction;

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    return 'et_transaction_view_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'et_transaction_view_form_' . $this->et_transaction->id();
  }

  /**
   * Set the EtTransaction of this form.
   *
   * @param \Drupal\expense_tracker\EtTransactionInterface $et_transaction
   *   The et_transaction that will be set in the form.
   */
  public function setEtTransaction(EtTransactionInterface $et_transaction) {
    $this->et_transaction = $et_transaction;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL, $view_mode = 'full') {
    // Add the et_transaction to the form.
    $form['et_transaction']['#type'] = 'value';
    $form['et_transaction']['#value'] = $this->et_transaction;

    $form['#view_mode'] = $view_mode;

    if ($this->showResults($this->et_transaction, $form_state)) {

      // Check if the user already voted. The form is still being built but
      // the Vote button won't be added so the submit callbacks will not be
      // called. Directly check for the request method and use the raw user
      // input.
      if ($request->isMethod('POST') && $this->et_transaction->hasUserVoted()) {
        $input = $form_state->getUserInput();
        if (isset($input['op']) && $input['op'] == $this->t('Vote')) {
          // If this happened, then the form submission was likely a cached page.
          // Force a session for this user so he can see the results.
          $this->messenger()->addError($this->t('Your vote for this et_transaction has already been submitted.'));
          $_SESSION['expense_tracker_vote'][$this->et_transaction->id()] = FALSE;
        }
      }

      $form['results'] = $this->showEtTransactionResults($this->et_transaction, $view_mode);

      // For all view modes except full and block (as block displays it as the
      // block title), display the title.
      if ($view_mode != 'full' && $view_mode != 'block') {
        $form['results']['#show_title'] = TRUE;
      }
    }
    else {
      $options = $this->et_transaction->getOptions();
      if ($options) {
        $form['choice'] = array(
          '#type' => 'radios',
          '#title' => t('Choices'),
          '#title_display' => 'invisible',
          '#options' => $options,
        );
      }
      $form['#theme'] = 'expense_tracker_vote';
      $form['#entity'] = $this->et_transaction;
      $form['#action'] = $this->et_transaction->toUrl()->setOption('query', \Drupal::destination()->getAsArray())->toString();
      // Set a flag to hide results which will be removed if we want to view
      // results when the form is rebuilt.
      $form_state->set('show_results', FALSE);

      // For all view modes except full and block (as block displays it as the
      // block title), display the title.
      if ($view_mode != 'full' && $view_mode != 'block') {
        $form['#show_title'] = TRUE;
      }

    }

    $form['actions'] = $this->actions($form, $form_state, $this->et_transaction);

    $form['#cache'] = array(
      'tags' => $this->et_transaction->getCacheTags(),
    );

    return $form;
  }

  /**
   * Ajax callback to replace the et_transaction form.
   */
  public function ajaxReplaceForm(array $form, FormStateInterface $form_state) {
    // Embed status message into the form.
    $form = ['messages' => ['#type' => 'status_messages']] + $form;
    /** @var \Drupal\Core\Render\RendererInterface $renderer */
    $renderer = \Drupal::service('renderer');
    // Render the form.
    $output = $renderer->renderRoot($form);

    $response = new AjaxResponse();
    $response->setAttachments($form['#attached']);

    // Replace the form completely and return it.
    return $response->addCommand(new ReplaceCommand('.et_transaction-view-form-' . $this->et_transaction->id(), $output));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  public function showResults(EtTransactionInterface $et_transaction, FormStateInterface $form_state) {
    $account = $this->currentUser();
    switch (TRUE) {
      // The "View results" button, when available, has been clicked.
      case $form_state->get('show_results'):
        return TRUE;

      // The et_transaction is closed.
      case ($et_transaction->isClosed()):
        return TRUE;

      // Anonymous user is trying to view a et_transaction they aren't allowed to vote in.
      case ($account->isAnonymous() && !$et_transaction->getAnonymousVoteAllow()):
        return TRUE;

      // The user has already voted.
      case ($et_transaction->hasUserVoted()):
        return TRUE;

      default:
        return FALSE;
    }
  }

  protected function actions(array $form, FormStateInterface $form_state, $et_transaction) {
    $actions = [];

    // Default ajax behavior, use the et_transaction URL for faster submission, this
    // requires that we explicitly provide the ajax_form query argument too in
    // the separate options key, as that replaces all options of the Url object.
    $ajax = [
      'callback' => '::ajaxReplaceForm',
      'url' => $this->et_transaction->toUrl(),
      'options' => ['query' => [FormBuilderInterface::AJAX_FORM_REQUEST => TRUE, 'view_mode' => $form['#view_mode']]]
    ];

    if ($this->showResults($et_transaction, $form_state)) {
      // Allow user to cancel their vote.
      if ($this->isCancelAllowed($et_transaction)) {
        $actions['#type'] = 'actions';
        $actions['cancel']['#type'] = 'submit';
        $actions['cancel']['#button_type'] = 'primary';
        $actions['cancel']['#value'] = t('Cancel vote');
        $actions['cancel']['#submit'] = array('::cancel');
        $actions['cancel']['#ajax'] = $ajax;
        $actions['cancel']['#weight'] = '0';
      }
      if (!$et_transaction->hasUserVoted() && $et_transaction->isOpen() && $et_transaction->getAnonymousVoteAllow()) {
        $actions['#type'] = 'actions';
        $actions['back']['#type'] = 'submit';
        $actions['back']['#button_type'] = 'primary';
        $actions['back']['#value'] = t('View et_transaction');
        $actions['back']['#submit'] = array('::back');
        $actions['back']['#ajax'] = $ajax;
        $actions['back']['#weight'] = '0';
      }
    }
    else {
      $actions['#type'] = 'actions';
      $actions['vote']['#type'] = 'submit';
      $actions['vote']['#button_type'] = 'primary';
      $actions['vote']['#value'] = t('Vote');
      $actions['vote']['#validate'] = array('::validateVote');
      $actions['vote']['#submit'] = array('::save');
      $actions['vote']['#ajax'] = $ajax;
      $actions['vote']['#weight'] = '0';

      // View results before voting.
      if ($et_transaction->result_vote_allow->value || $this->currentUser()->hasPermission('view et_transaction results')) {
        $actions['result']['#type'] = 'submit';
        $actions['result']['#button_type'] = 'primary';
        $actions['result']['#value'] = t('View results');
        $actions['result']['#submit'] = array('::result');
        $actions['result']['#ajax'] = $ajax;
        $actions['result']['#weight'] = '1';
      }
    }

    return $actions;
  }

  /**
   * Display a themed et_transaction results.
   *
   * @param \Drupal\expense_tracker\EtTransactionInterface $et_transaction
   *   The et_transaction entity.
   * @param bool $block
   *   (optional) TRUE if a et_transaction should be displayed in a block. Defaults to
   *   FALSE.
   *
   * @return array $output
   */
  function showEtTransactionResults(EtTransactionInterface $et_transaction, $view_mode = 'default', $block = FALSE) {

    // Ensure that a page that shows et_transaction results can not be cached.
    \Drupal::service('page_cache_kill_switch')->trigger();

    $total_votes = 0;
    foreach ($et_transaction->getVotes() as $vote) {
      $total_votes += $vote;
    }

    $options = $et_transaction->getOptions();
    $et_transaction_results = array();
    foreach ($et_transaction->getVotes() as $pid => $vote) {
      $percentage = round($vote * 100 / max($total_votes, 1));
      $display_votes = (!$block) ? ' (' . \Drupal::translation()
          ->formatPlural($vote, '1 vote', '@count votes') . ')' : '';

      $et_transaction_results[] = array(
        '#theme' => 'et_transaction_meter',
        '#choice' => $options[$pid],
        '#display_value' => t('@percentage%', array('@percentage' => $percentage)) . $display_votes,
        '#min' => 0,
        '#max' => $total_votes,
        '#value' => $vote,
        '#percentage' => $percentage,
        '#attributes' => array('class' => array('bar')),
        '#et_transaction' => $et_transaction,
      );
    }

    /** @var \Drupal\et_transaction\EtTransactionVoteStorageInterface $vote_storage */
    $vote_storage = \Drupal::service('expense_tracker_vote.storage');
    $user_vote = $vote_storage->getUserVote($et_transaction);

    $output = array(
      '#theme' => 'et_transaction_results',
      '#raw_title' => $et_transaction->label(),
      '#results' => $et_transaction_results,
      '#votes' => $total_votes,
      '#block' => $block,
      '#pid' => $et_transaction->id(),
      '#et_transaction' => $et_transaction,
      '#view_mode' => $view_mode,
      '#vote' => isset($user_vote['chid']) ? $user_vote['chid'] : NULL,
    );

    return $output;
  }



  /**
   * Cancel vote submit function.
   *
   * @param array $form
   *   The previous form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function cancel(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\et_transaction\EtTransactionVoteStorageInterface $vote_storage */
    $vote_storage = \Drupal::service('expense_tracker_vote.storage');
    $vote_storage->cancelVote($this->et_transaction, $this->currentUser());
    \Drupal::logger('et_transaction')->notice('%user\'s vote in EtTransaction #%et_transaction deleted.', array(
      '%user' => $this->currentUser()->id(),
      '%et_transaction' => $this->et_transaction->id(),
    ));
    $this->messenger()->addMessage($this->t('Your vote was cancelled.'));

    // In case of an ajax submission, trigger a form rebuild so that we can
    // return an updated form through the ajax callback.
    if ($this->getRequest()->query->get('ajax_form')) {
      $form_state->setRebuild(TRUE);
    }
  }

  /**
   * View vote results submit function.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function result(array $form, FormStateInterface $form_state) {
    $form_state->set('show_results', TRUE);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Back to et_transaction view submit function.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function back(array $form, FormStateInterface $form_state) {
    $form_state->set('show_results', FALSE);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Save a user's vote submit function.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function save(array $form, FormStateInterface $form_state) {
    $options = array();
    $options['chid'] = $form_state->getValue('choice');
    $options['uid'] = $this->currentUser()->id();
    $options['pid'] = $form_state->getValue('et_transaction')->id();
    $options['hostname'] = \Drupal::request()->getClientIp();
    $options['timestamp'] = \Drupal::time()->getRequestTime();
    // Save vote.
    /** @var \Drupal\et_transaction\EtTransactionVoteStorage $vote_storage */
    $vote_storage = \Drupal::service('expense_tracker_vote.storage');
    $vote_storage->saveVote($options);
    $this->messenger()->addMessage($this->t('Your vote has been recorded.'));

    if ($this->currentUser()->isAnonymous()) {
      // The vote is recorded so the user gets the result view instead of the
      // voting form when viewing the et_transaction. Saving a value in $_SESSION has the
      // convenient side effect of preventing the user from hitting the page
      // cache. When anonymous voting is allowed, the page cache should only
      // contain the voting form, not the results.
      $_SESSION['expense_tracker_vote'][$form_state->getValue('et_transaction')->id()] = $form_state->getValue('choice');
    }

    // In case of an ajax submission, trigger a form rebuild so that we can
    // return an updated form through the ajax callback.
    if ($this->getRequest()->query->get('ajax_form')) {
      $form_state->setRebuild(TRUE);
    }

    // No explicit redirect, so that we stay on the current page, which might
    // be the et_transaction form or another page that is displaying this et_transaction, for
    // example as a block.
  }

  /**
   * Validates the vote action.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function validateVote(array &$form, FormStateInterface $form_state) {
    if (!$form_state->hasValue('choice')) {
      $form_state->setErrorByName('choice', $this->t('Your vote could not be recorded because you did not select any of the choices.'));
    }
  }

  /**
   * Checks if the current user is allowed to cancel on the given et_transaction.
   *
   * @param \Drupal\expense_tracker\EtTransactionInterface $et_transaction
   *
   * @return bool
   *   TRUE if the user can cancel.
   */
  protected function isCancelAllowed(EtTransactionInterface $et_transaction) {
    // Allow access if the user has voted.
    return $et_transaction->hasUserVoted()
      // And the et_transaction allows to cancel votes.
      && $et_transaction->getCancelVoteAllow()
      // And the user has the cancel own vote permission.
      && $this->currentUser()->hasPermission('cancel own vote')
      // And the user is authenticated or his session contains the voted flag.
      && (\Drupal::currentUser()->isAuthenticated() || !empty($_SESSION['expense_tracker_vote'][$et_transaction->id()]))
      // And et_transaction is open.
      && $et_transaction->isOpen();
  }

}

==> expense_tracker-1.3.3.tar.gz: expense_tracker/src/Form/EtTransactionViewForm.php <==
<?php

namespace Drupal\expense_tracker\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\BaseFormIdInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\expense_tracker\EtTransactionInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Displays banned IP addresses.
 */
class EtTransactionViewForm extends FormBase implements BaseFormIdInterface {

  /**
   * The EtTransaction of the form.
   *
   * @var \Drupal\expense_tracker\EtTransactionInterface
   */
  protected $et_transaction;

  /**
   * {@inheritdoc}
   */
  public function getBaseFormId() {
    return 'et_transaction_view_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'et_transaction_view_form_' . $this->et_transaction->id();
  }

  /**
   * Set the EtTransaction of this form.
   *
   * @param \Drupal\expense_tracker\EtTransactionInterface $et_transaction
   *   The et_transaction that will be set in the form.
   */
  public function setEtTransaction(EtTransactionInterface $et_transaction) {
    $this->et_transaction = $et_transaction;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL, $view_mode = 'full') {
    // Add the et_transaction to the form.
    $form['et_transaction']['#type'] = 'value';
    $form['et_transaction']['#value'] = $this->et_transaction;

    $form['#view_mode'] = $view_mode;

    if ($this->showResults($this->et_transaction, $form_state)) {

      // Check if the user already. The form is still being built but
      // the Vote button won't be added so the submit callbacks will not be
      // called. Directly check for the request method and use the raw user
      // input.
      if ($request->isMethod('POST') && $this->et_transaction->hasUserVoted()) {
        $input = $form_state->getUserInput();
        if (isset($input['op']) && $input['op'] == $this->t('Vote')) {
          // If this happened, then the form submission was likely a cached page.
          // Force a session for this user so he can see the results.
          $this->messenger()->addError($this->t('Your transaction for this et_transaction has already been submitted.'));
          $_SESSION['expense_tracker'][$this->et_transaction->id()] = FALSE;
        }
      }

      $form['results'] = $this->showEtTransactionResults($this->et_transaction, $view_mode);

      // For all view modes except full and block (as block displays it as the
      // block title), display the title.
      if ($view_mode != 'full' && $view_mode != 'block') {
        $form['results']['#show_title'] = TRUE;
      }
    }
    else {
      $options = $this->et_transaction->getOptions();
      if ($options) {
        $form['choice'] = array(
          '#type' => 'radios',
          '#title' => t('Choices'),
          '#title_display' => 'invisible',
          '#options' => $options,
        );
      }
      $form['#theme'] = 'expense_tracker';
      $form['#entity'] = $this->et_transaction;
      $form['#action'] = $this->et_transaction->toUrl()->setOption('query', \Drupal::destination()->getAsArray())->toString();
      // Set a flag to hide results which will be removed if we want to view
      // results when the form is rebuilt.
      $form_state->set('show_results', FALSE);

      // For all view modes except full and block (as block displays it as the
      // block title), display the title.
      if ($view_mode != 'full' && $view_mode != 'block') {
        $form['#show_title'] = TRUE;
      }

    }

    $form['actions'] = $this->actions($form, $form_state, $this->et_transaction);

    $form['#cache'] = array(
      'tags' => $this->et_transaction->getCacheTags(),
    );

    return $form;
  }

  /**
   * Ajax callback to replace the et_transaction form.
   */
  public function ajaxReplaceForm(array $form, FormStateInterface $form_state) {
    // Embed status message into the form.
    $form = ['messages' => ['#type' => 'status_messages']] + $form;
    /** @var \Drupal\Core\Render\RendererInterface $renderer */
    $renderer = \Drupal::service('renderer');
    // Render the form.
    $output = $renderer->renderRoot($form);

    $response = new AjaxResponse();
    $response->setAttachments($form['#attached']);

    // Replace the form completely and return it.
    return $response->addCommand(new ReplaceCommand('.et_transaction-view-form-' . $this->et_transaction->id(), $output));
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  public function showResults(EtTransactionInterface $et_transaction, FormStateInterface $form_state) {
    $account = $this->currentUser();
    switch (TRUE) {
      // The "View results" button, when available, has been clicked.
      case $form_state->get('show_results'):
      return TRUE;

      // The et_transaction is closed.
      case ($et_transaction->isClosed()):
      return TRUE;

      default:
      return FALSE;
    }
  }

  protected function actions(array $form, FormStateInterface $form_state, $et_transaction) {
    $actions = [];

    // Default ajax behavior, use the et_transaction URL for faster submission, this
    // requires that we explicitly provide the ajax_form query argument too in
    // the separate options key, as that replaces all options of the Url object.
    $ajax = [
      'callback' => '::ajaxReplaceForm',
      'url' => $this->et_transaction->toUrl(),
      'options' => ['query' => [FormBuilderInterface::AJAX_FORM_REQUEST => TRUE, 'view_mode' => $form['#view_mode']]]
    ];

    if ($this->showResults($et_transaction, $form_state)) {
      // Allow user to cancel their transaction.
      if ($this->isCancelAllowed($et_transaction)) {
        $actions['#type'] = 'actions';
        $actions['cancel']['#type'] = 'submit';
        $actions['cancel']['#button_type'] = 'primary';
        $actions['cancel']['#value'] = t('Cancel transaction');
        $actions['cancel']['#submit'] = array('::cancel');
        $actions['cancel']['#ajax'] = $ajax;
        $actions['cancel']['#weight'] = '0';
      }
    }
    else {
      $actions['#type'] = 'actions';
      $actions['transaction']['#type'] = 'submit';
      $actions['transaction']['#button_type'] = 'primary';
      $actions['transaction']['#submit'] = array('::save');
      $actions['transaction']['#ajax'] = $ajax;
      $actions['transaction']['#weight'] = '0';

      // View results before voting.
      if ($et_transaction->result_allow->value || $this->currentUser()->hasPermission('view et_transaction results')) {
        $actions['result']['#type'] = 'submit';
        $actions['result']['#button_type'] = 'primary';
        $actions['result']['#value'] = t('View results');
        $actions['result']['#submit'] = array('::result');
        $actions['result']['#ajax'] = $ajax;
        $actions['result']['#weight'] = '1';
      }
    }

    return $actions;
  }

  /**
   * Display a themed et_transaction results.
   *
   * @param \Drupal\expense_tracker\EtTransactionInterface $et_transaction
   *   The et_transaction entity.
   * @param bool $block
   *   (optional) TRUE if a et_transaction should be displayed in a block. Defaults to
   *   FALSE.
   *
   * @return array $output
   */
  function showEtTransactionResults(EtTransactionInterface $et_transaction, $view_mode = 'default', $block = FALSE) {}



  /**
   * Cancel transaction submit function.
   *
   * @param array $form
   *   The previous form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function cancel(array $form, FormStateInterface $form_state) {}

  /**
   * View transaction results submit function.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function result(array $form, FormStateInterface $form_state) {
    $form_state->set('show_results', TRUE);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Back to et_transaction view submit function.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function back(array $form, FormStateInterface $form_state) {
    $form_state->set('show_results', FALSE);
    $form_state->setRebuild(TRUE);
  }

  /**
   * Save a user's transaction submit function.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function save(array $form, FormStateInterface $form_state) {}

  /**
   * Validates the transaction action.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function validateVote(array &$form, FormStateInterface $form_state) {}

  /**
   * Checks if the current user is allowed to cancel on the given et_transaction.
   *
   * @param \Drupal\expense_tracker\EtTransactionInterface $et_transaction
   *
   * @return bool
   *   TRUE if the user can cancel.
   */
  protected function isCancelAllowed(EtTransactionInterface $et_transaction) {}

}
